#ifndef PLUGINPROVER_H
#define PLUGINPROVER_H

#include "dlfcn.h"
#include <string>

using namespace std;

namespace AMDiS {
template< typename DesiredCreator >
struct PluginProvider { 
	typedef DesiredCreator* Creator();
	typedef void Remover(DesiredCreator* );

	string createName;
	string removeName;

	Remover* remover;

	PluginProvider(string c, string r):
		createName(c),
		removeName(r),
		remover(NULL)
	{}

	enum Error { NOLIBRARY, NOCREATOR, NOREMOVER };
	struct Exception {
		Error e;
		string message;
		Exception(Error e, string m):
			e(e),message(m)
		{}
	};

	DesiredCreator* create(string libname) {
		void* library=dlopen(libname.c_str(), RTLD_LAZY);
		if(library==NULL) {
			throw Exception(NOLIBRARY, dlerror());
		}

		dlerror();

		Creator* creator=(Creator*) (dlsym(library, createName.c_str()));
		if(creator == NULL) {
			throw Exception(NOCREATOR, dlerror());
		}

		remover=(Remover*) (dlsym(library, removeName.c_str()));
		if(remover == NULL) {
			throw Exception(NOREMOVER, dlerror());
		}

		DesiredCreator* creatingCreator=creator();
		return creatingCreator;
	}

	void remove(DesiredCreator* dc) {
		if(remover == NULL) {
			throw Exception(NOREMOVER, "the remover was not set\n");
		}
		remover(dc);
	}
};

//typedef OEMSolverCreator* createSolverCreator();
//typedef void removeSolverCreator(OEMSolverCreator*);
struct SolverPluginProvider : PluginProvider< OEMSolverCreator > {
	SolverPluginProvider():
		PluginProvider< OEMSolverCreator >("createSolverCreator","removeSolverCreator")
	{}
};
}
#endif
