#include "BunnyProject.h"
#include "AdaptStationary.h"

/// RHS function
class F : public AbstractFunction<double, WorldVector<double> >
{
public:
  F(int degree) : AbstractFunction<double, WorldVector<double> >(degree) {}

  /// Implementation of AbstractFunction::operator().
  double operator()(const WorldVector<double>& x) const 
  {
    return -2 * x[0];
  }
};

/// boundary
class G : public AbstractFunction<double, WorldVector<double> >
{
public:

  /// Implementation of AbstractFunction::operator().
  double operator()(const WorldVector<double>& x) const 
  {
    return 10000.0;
  }
};

Bunnydemo::Bunnydemo():
  bunny("bunny"),
  ballCenter(NULL),
  adaptInfo(NULL),
  adapt(NULL),
  matrixOperator(NULL),
  rhsOperator(NULL)
{}

Bunnydemo::~Bunnydemo() {
  if(ballCenter != NULL)
    delete ballCenter;
  if(adaptInfo != NULL)
    delete adaptInfo;
  if(adapt != NULL)
    delete adapt;
  if(matrixOperator != NULL)
    delete matrixOperator;
  if(rhsOperator != NULL)
    delete rhsOperator;
}

void Bunnydemo::create(string& filename) {
   // ===== init parameters =====
  Parameters::init(false, filename);

  // ===== create projection =====
  ballCenter = new WorldVector< double > ();
  ballCenter->set(0.0);
  new BallProject(1, VOLUME_PROJECTION, *ballCenter, 1.0);

  // ===== create and init the scalar problem ===== 
  bunny.initialize(INIT_ALL);

  // === create adapt info ===
  adaptInfo = new AdaptInfo("bunny->adapt");

  // === create adapt ===
  adapt = new AdaptStationary("bunny->adapt", &bunny, adaptInfo);
  
  // ===== create matrix operator =====
  matrixOperator = new Operator(bunny.getFeSpace());
  matrixOperator->addSecondOrderTerm(new Laplace_SOT);
  bunny.addMatrixOperator(matrixOperator);

  // ===== create rhs operator =====
  rhsOperator = new Operator(bunny.getFeSpace());

  int degree = bunny.getFeSpace()->getBasisFcts()->getDegree();

  rhsOperator->addZeroOrderTerm(new CoordsAtQP_ZOT(new F(degree)));
  bunny.addVectorOperator(rhsOperator);

}

int Bunnydemo::solve(SolutionInformation& info) {
  int retCode = adapt->adapt();
  info.dofVec = bunny.getSolution();
  return retCode;
}

